/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

// for slots manipulation
import java.util.Collection;
import java.util.Map;

/**
 * MOF 2.0 Element (UML Element actually).
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Element.java 1225 2011-04-19 22:00:46Z guard $
 * @todo #2:2 This class has to be renamed to Instance.
 */
public interface Element {

    /**
     * Get instance name.
     * @return The name
     */
    String name();

    /**
     * Get class name.
     * @return The name
     */
    String type();

    /**
     * This element contains given slot name?
     * @param key Slot name
     * @return Contains?
     */
    boolean has(final String key);

    /**
     * Get slot by name.
     * @param key Slot name
     * @return The value
     */
    String get(final String key);

    /**
     * Get slot by name, with the given class.
     * @param <T> Type of return value
     * @param key Slot name
     * @param type The class to use
     * @return The value
     */
    <T> T get(final String key, final Class<T> type);

    /**
     * This slot refers to another element?
     * @param key Slot name
     * @param another Another element
     * @return Yes or no?
     */
    boolean refsTo(final String key, final Element another);

    /**
     * Set slot value.
     * @param key Slot name
     * @param value The value
     */
    void put(final String key, final String value);

    /**
     * Get all slot values with the given name.
     * @param key Slot name
     * @return The value
     */
    Collection<String> all(final String key);

    /**
     * Set full collection of slots.
     * @return The collection
     */
    Collection<Map.Entry<String, String>> entrySet();

}
