/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

// for I/O problems
import java.io.IOException;

/**
 * Reader of meta information. The reader shall be able to initialize
 * itself with an instance of {@link Rempler} and then load necessary
 * meta information from the code base and transform the {@link Model}
 * provided.
 *
 * <p>Every reader has to be designed for multiple consecutive calls
 * to its {@link #read(Model)} method. After every call the reader
 * has to return TRUE of FALSE, which will mean that it has to be
 * called again or not.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Reader.java 555 2011-03-12 17:23:39Z yegor256@yahoo.com $
 */
public interface Reader {

    /**
     * Save the {@link Rempler}, a runner of the Reader.
     * @param rempler The Rempler
     */
    void setRempler(final Rempler rempler);

    /**
     * Read meta information and transform the {@link Model} provided.
     * @param model The MOF {@link Model} to use for assertions
     * @return Shall we call this reader again?
     * @throws IOException If I/O problem occurs
     */
    boolean read(final Model model) throws IOException;

}
