/**
 * @version $Id: ROMTest.java 945 2011-03-27 11:05:08Z yegor256@yahoo.com $
 */
package com.rempl.api;

import org.junit.*;
import static org.junit.Assert.*;
import com.rempl.tk.ResourceLoader;
import java.io.*;
import org.apache.commons.io.FileUtils;

public class ROMTest {

    @Test
    public void testGlobalElements() throws Exception {
        ROM rom = new ROM(ResourceLoader.find("complete.xml"));
        assertEquals(2, rom.readers().size());
        assertEquals(1, rom.dependencies().size());
        assertEquals(1, rom.reporters().size());
        assertEquals(
            ".",
            rom.configuration()
            .get("/configuration/sources/directory/text()", String.class)
        );
        assertTrue(rom.hasWorker());
        assertEquals("java.lang.String", rom.getWorker().getClass().getName());
    }

    @Test
    public void testFindsReadersAndReporters() throws Exception {
        ROM rom = new ROM(ResourceLoader.find("complete.xml"));
        for (Artifact dep : rom.readers()) {
            assertTrue(
                "Invalid artifact found: " + dep,
                dep.toString().startsWith("com.rempl.readers:")
            );
        }
        for (Artifact dep : rom.reporters()) {
            assertTrue(
                "Invalid reporter found: " + dep,
                dep.toString().startsWith("com.rempl.reporters:")
            );
        }
    }

    @Test
    public void testWorksWithEmptyROM() throws Exception {
        ROM rom = new ROM(ResourceLoader.find("empty.xml"));
        assertEquals(0, rom.readers().size());
        assertEquals(0, rom.dependencies().size());
        assertEquals(0, rom.reporters().size());
        assertFalse(rom.hasWorker());
    }

    @Test
    public void testWorksWithEmptyROMAndAlternativeConstructor() throws Exception {
        ROM rom = new ROM(
            FileUtils.readFileToString(ResourceLoader.find("empty.xml"))
        );
        assertEquals(0, rom.readers().size());
    }

    @Test(expected = ROMException.class)
    public void testDetectsInvalidWorkerClass() throws Exception {
        ROM rom = new ROM(ResourceLoader.find("invalid-worker.xml"));
        assertTrue(rom.hasWorker());
        rom.getWorker();
    }

}
