/**
 * @version $Id: ROMUtilsTest.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.api;

import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;
import org.junit.*;
import org.junit.rules.*;
import static org.junit.Assert.*;
import com.rempl.tk.ResourceLoader;
import java.io.File;
import java.io.StringReader;
import org.apache.commons.io.FileUtils;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPathConstants;

public class ROMUtilsTest {

    private ROMUtils utils = new ROMUtils();

    @Rule
    public ErrorCollector errors = new ErrorCollector();

    @Rule
    public TemporaryFolder tempFolder = new TemporaryFolder();

    @Test
    public void testLoadsValidXMLDocuments() throws Exception {
        String[] resources = { "simple", "full" };
        for (String resource : resources) {
            String name = "valid/" + resource + ".xml";
            try {
                utils.parse(ResourceLoader.find(name));
                utils.parse(FileUtils.readFileToString(ResourceLoader.find(name)));
            } catch (ROMException ex) {
                this.errors.addError(new Exception("Exception in " + name, ex));
            }
        }
    }

    @Test
    public void testLoadsInvalidXMLDocuments() throws Exception {
        String[] resources = { "no-namespace", "duplicate-name", "wrong-formats" };
        for (String resource : resources) {
            String name = "invalid/" + resource + ".xml";
            try {
                utils.parse(ResourceLoader.find(name));
                this.errors.addError(new ROMException("Document " + name + " is valid, why?"));
            } catch (ROMException ex) {
                Logger.debug(this, "exception full text: " + ex.getMessage());
            }
        }
    }

    @Test(expected = ROMException.class)
    public void testLoadsNonExistingDocument() throws Exception {
        this.utils.parse(new File("file-is-absent-for-sure.xml"));
    }

    @Test(expected = ROMException.class)
    public void testLoadsNonXMLDocument() throws Exception {
        File temp = this.tempFolder.newFile("temp.xml");
        FileUtils.writeStringToFile(temp, "some data which is not XML!");
        this.utils.parse(temp);
    }

    @Test
    public void testXPathWithSingleNode() throws Exception {
        Node node = (Node) this.utils.xpath(
            "/*/foo/bar/text()",
            this.document(),
            XPathConstants.NODE
        );
        assertNotNull("Node not found, why?", node);
        assertEquals("test", node.getNodeValue());
    }

    @Test
    public void testXPathAbsentNode() throws Exception {
        Node node = (Node) this.utils.xpath(
            "/*/absent-node/text()",
            this.document(),
            XPathConstants.NODE
        );
        assertNull(node);
    }

    @Test(expected = RuntimeProblem.class)
    public void testWithInvalidXPathExpression() throws Exception {
        this.utils.xpath(
            "invalid xpath expression",
            this.document(),
            XPathConstants.NODE
        );
    }

    @Test
    public void testTextExistingNode() throws Exception {
        String txt = this.utils.text(
            "/*/foo/bar/text()",
            this.document()
        );
        assertEquals("test", txt);
    }

    @Test(expected = RuntimeProblem.class)
    public void testTextNonExistingNode() throws Exception {
        this.utils.text(
            "/*/non-existing-node/text()",
            this.document()
        );
    }

    private Document document() throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        String xml = "<root><foo><bar>test</bar></foo></root>";
        return builder.parse(new InputSource(new StringReader(xml)));
    }

}
