/**
 * @version $Id: ReaderTest.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.api;

import com.ymock.util.Logger;
import org.junit.*;
import static org.junit.Assert.*;
import com.rempl.tk.*;
import com.rempl.impl.*;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.io.FilenameUtils;
import java.util.*;

public class ReaderTest {

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Reader")
                .methodExists("setRempler(com.rempl.api.Rempler)")
                    .back() // to class
                .methodExists("read(com.rempl.api.Model)")
                    .returns("boolean")
                    .back() // to class
                    ;
    }

    @Test
    public void testReadsThisProjectMetaInfo() throws Exception {
        RemplFactory factory = Rempl.getFactory();
        factory.setLoader(new LoaderImpl());
        ROM rom = new ROM(ResourceLoader.find("this-project.xml"));
        Rempler rempler = factory.makeRempler(rom);
        rempler.configuration().set("sources/includes/include", "**/*.java");
        Model model = new ModelImpl();
        Reader reader = new ReaderImpl();
        reader.setRempler(rempler);
        reader.read(model);
        Collection<Element> elements = model.query("context Artifact inv: true");
        assertTrue("Model is empty, why?", elements.size() > 0);
        List<String> files = new ArrayList<String>();
        for (Element element : elements) {
            assertTrue(element.has("filename"));
            String fname = (String) element.get("filename");
            files.add(fname);
            assertEquals("java", FilenameUtils.getExtension(fname));
        }
        Logger.debug(this, "files discovered: " + StringUtils.join(files, ", "));
    }

}
