/**
 * @version $Id: RemplerTest.java 776 2011-03-18 22:35:07Z yegor256@yahoo.com $
 */
package com.rempl.api;

import org.junit.*;
import static org.junit.Assert.*;
import com.rempl.tk.*;
import com.rempl.impl.*;
import org.apache.commons.lang.StringUtils;
import java.util.*;

public class RemplerTest {

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Rempler")
                .methodExists("read()")
                    .returns("com.rempl.api.Model")
                    .back() // to class
                    ;
    }

    @Test
    public void testReadsThisProjectMetaInfo() throws Exception {
        RemplFactory factory = Rempl.getFactory();
        factory.setLoader(new LoaderImpl());
        ROM rom = new ROM(ResourceLoader.find("this-project.xml"));
        Rempler rempler = factory.makeRempler(rom);
        rempler.configuration().set("sources/includes/include", "**/*.java");
        Model model = rempler.read();
        Collection<Element> elements = model.query("context Artifact inv: true");
        assertTrue("Model is empty, why?", elements.size() > 0);
    }

    @Test
    public void testManipulateConfiguration() throws Exception {
        Rempler rempler = new RemplerImpl();
        rempler.configure(new ROMConfiguration(null));
        rempler.configuration().set("sources/includes/include", "**/*.java");
        assertEquals("**/*.java", rempler.configuration().get("/*/sources/includes/include/text()"));
    }

    @Test
    public void testMakesEmptyModel() throws Exception {
        Rempler rempler = new RemplerImpl();
        assertTrue(rempler.emptyModel() instanceof Model);
    }

}
