/**
 * @version $Id: ReporterTest.java 712 2011-03-16 22:05:17Z yegor256@yahoo.com $
 */
package com.rempl.api;

import org.junit.*;
import org.junit.rules.*;
import static org.junit.Assert.*;
import com.rempl.tk.*;
import com.rempl.impl.*;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.*;
import java.util.*;
import java.io.File;

public class ReporterTest {

    @Rule
    public TemporaryFolder tempFolder = new TemporaryFolder();

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Reporter")
                .methodExists("setRempler(com.rempl.api.Rempler)")
                    .back() // to class
                .methodExists("report(com.rempl.api.Model, java.io.File)")
                    .back() // to class
                    ;
    }

    @Test
    public void testReportsThisProject() throws Exception {
        RemplFactory factory = Rempl.getFactory();
        factory.setLoader(new LoaderImpl());
        ROM rom = new ROM(ResourceLoader.find("this-project.xml"));
        Rempler rempler = factory.makeRempler(rom);
        Model model = rempler.read();
        Reporter reader = new ReporterImpl();
        reader.setRempler(rempler);
        File folder = this.tempFolder.newFolder("report");
        reader.report(model, folder);
        Iterator<File> files = FileUtils.iterateFiles(
            folder,
            TrueFileFilter.INSTANCE,
            TrueFileFilter.INSTANCE
        );
        Collection<String> names = new ArrayList<String>();
        while (files.hasNext()) {
            names.add(files.next().getPath());
        }
        assertTrue("Report folder is empty, why?", names.size() > 0);
    }

}
