/**
 * @version $Id: ModelImpl.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.impl;

import com.rempl.api.Element;
import com.rempl.api.Model;
import com.rempl.impl.model.QueryLexer;
import com.rempl.impl.model.QueryParser;
import com.rempl.impl.model.TransformLexer;
import com.rempl.impl.model.TransformParser;
import com.ymock.util.Logger;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import com.ymock.util.RuntimeProblem;
import org.antlr.runtime.ANTLRStringStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.TokenStream;
import org.apache.commons.io.IOUtils;

public class ModelImpl implements Model {

    Collection<Element> elements = new ArrayList<Element>();

    @Override
    public void transform(final String query) {
        Logger.trace(this, "#transform(%s)", query);
        CharStream input = new ANTLRStringStream(query);
        TransformLexer lexer = new TransformLexer(input);
        TokenStream tokens = new CommonTokenStream(lexer);
        TransformParser parser = new TransformParser(tokens);
        Collection<Element> elements;
        try {
            elements = parser.elements();
        } catch (org.antlr.runtime.RecognitionException ex) {
            throw RuntimeProblem.make(ex);
        }
        Logger.debug(this, "#transform(..): loaded %d elements", elements.size());
        for (Element e : elements) {
            this.elements.add(e);
        }
    }

    @Override
    public Collection<Element> query(final String query) {
        Logger.trace(this, "#query(%s)", query);
        CharStream input = new ANTLRStringStream(query);
        QueryLexer lexer = new QueryLexer(input);
        TokenStream tokens = new CommonTokenStream(lexer);
        QueryParser parser = new QueryParser(tokens);
        String context;
        try {
            context = parser.context();
        } catch (org.antlr.runtime.RecognitionException ex) {
            throw RuntimeProblem.make(ex);
        }
        Collection<Element> list = new ArrayList<Element>();
        for (Element e : this.elements) {
            if (e.type().equals(context)) {
                list.add(e);
            }
        }
        return list;
    }

    @Override
    public void save(final Writer writer) throws IOException {
        Logger.trace(this, "#save(writer)");
        for (Element e : this.elements) {
            StringBuffer buffer = new StringBuffer();
            if (e.name() != null) {
                buffer.append(e.name());
            }
            buffer.append(":" + e.type() + " { ");
            for (Map.Entry<String, String> pair : e.entrySet()) {
                buffer.append(pair.getKey() + ": \"" + pair.getValue() + "\"; ");
            }
            buffer.append(" };\n");
            writer.write(buffer.toString());
        }
    }

    @Override
    public void load(final java.io.Reader reader) throws IOException {
        Logger.trace(this, "#load(reader)");
        String xmi = IOUtils.toString(reader);
        this.transform(xmi);
    }

}
