/**
 * @version $Id: RemplFactoryImpl.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.impl;

import com.rempl.api.*;
import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;

public class RemplFactoryImpl implements RemplFactory {

    private Loader loader;

    @Override
    public void setLoader(final Loader ldr) {
        Logger.trace(this, "#setLoader(%s)", ldr.getClass().getName());
        this.loader = ldr;
    }

    @Override
    public void resetLoader() {
        Logger.trace(this, "#resetLoader()");
        this.loader = null;
    }

    @Override
    public Loader getLoader() {
        if (this.loader == null) {
            this.loader = new LoaderImpl();
            Logger.trace(
                this,
                "#getLoader(): set implicitly to %s",
                this.loader.getClass().getName()
            );
        }
        Logger.trace(this, "#getLoader(): %s", this.loader.getClass().getName());
        return this.loader;
    }

    @Override
    public Rempler makeRempler(final ROM rom) {
        Logger.trace(this, "#makeRempler()");
        Rempler rempler = new RemplerImpl();
        rempler.configure(this.load(rempler, rom));
        return rempler;
    }

    private ROMConfiguration load(Rempler rempler, ROM rom) {
        try {
            ROMConfiguration config = new ROMConfiguration(null);
            for (Artifact dependency : rom.dependencies()) {
                if (this.loader == null) {
                    Logger.warn(this, "Ignoring %s since loader is not set", dependency);
                    continue;
                }
                ROM dep = this.loader.load(dependency);
                config.append(this.load(rempler, dep)); // a recursive call to itself
                if (dep.hasWorker()) {
                    rempler.append((Reader) dep.getWorker());
                }
            }
            config.append(rom.configuration());
            return config;
        } catch (ROMException ex) {
            throw RuntimeProblem.make(ex);
        } catch (LoadingException ex) {
            throw RuntimeProblem.make(ex);
        }
    }

}
